/*
 * S2E Selective Symbolic Execution Framework
 *
 * Copyright (c) 2010, Dependable Systems Laboratory, EPFL
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Dependable Systems Laboratory, EPFL nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE DEPENDABLE SYSTEMS LABORATORY, EPFL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Currently maintained by:
 *    Volodymyr Kuznetsov <vova.kuznetsov@epfl.ch>
 *    Vitaly Chipounov <vitaly.chipounov@epfl.ch>
 *
 * All contributors are listed in S2E-AUTHORS file.
 *
 */


/*
 * This copy of s2e.h was modified for use in the Microsoft Visual Studio 
 * compiler (cl.exe). This required rewriting the inline assembly as
 * well as moving variable declarations to top of functions and defining
 * types that previously found in linux inttypes.h.
 *
 * Conversion done by Michael A. Kaplan at Applied Communication Sciences
 * Email: mkaplan@appcomsci.com 
 *
 */

#ifndef _S2E_W32_H_
#define _S2E_W32_H_

#ifdef USER_APP
#include <windows.h>
#include <stdio.h>
#include <string.h>
#endif

// Replacement to inttypes.h on linux
typedef unsigned __int32 uint32_t;
typedef unsigned __int64 uint64_t;

/** Forces the read of every byte of the specified string.
  * This makes sure the memory pages occupied by the string are paged in
  * before passing them to S2E, which can't page in memory by itself. */
static _inline void __s2e_touch_string(volatile const char *string)
{
    while(*string) {
        ++string;
    }
}

static _inline void __s2e_touch_buffer(volatile void *buffer, unsigned size)
{
    unsigned i;
    volatile char *b = (volatile char *)buffer;
    for (i=0; i<size; ++i) {
        *b; ++b;
    }
}

/** Get S2E version or 0 when running without S2E. */
static _inline int s2e_version()
{
	
    int version;
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : "=a" (version)  : "a" (0)
     * );
	 *
	 */
	_asm {
		mov eax, 0

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		mov version, eax
	}
	return version;
}

/** Print message to the S2E log. */
static _inline void s2e_message(const char* message)
{
    __s2e_touch_string(message);
    /* 
	 * Original GCC inline assembly left for reference
	 *
	 *__asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x10, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : : "a" (message)
     * );
	 *
	 */
	_asm { 
		mov eax, message
		
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x10
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Print warning to the S2E log and S2E stdout. */
static _inline void s2e_warning(const char* message)
{
    __s2e_touch_string(message);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x10, 0x01, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : : "a" (message)
     * );
	 *
	 */
	_asm {
		mov eax, message

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x10
		__emit 0x01
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Print symbolic expression to the S2E log. */
static _inline void s2e_print_expression(const char* name, int expression)
{
    __s2e_touch_string(name);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x07, 0x01, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : : "a" (expression), "c" (name)
     * );
	 *
	 */
	_asm {
		mov eax, expression
		mov ecx, name

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x07
		__emit 0x01
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Enable forking on symbolic conditions. */
static _inline void s2e_enable_forking(void)
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x09, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     * );
	 *
	 */
	_asm {
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x09
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Disable forking on symbolic conditions. */
static _inline void s2e_disable_forking(void)
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x0a, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     * );
	 *
	 */
	_asm {
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x0a
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Yield the current state */
static _inline void s2e_yield(void)
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x0F, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     * );
	 *
	 */
	_asm{
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x0F
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Get the current execution path/state id. */
static _inline unsigned s2e_get_path_id(void)
{
    unsigned id;
	/* 
	 * Original GCC inline assembly left for reference
	 *
     *__asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x05, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : "=a" (id)
     * );
	 *
	 */
	_asm {
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x05
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		mov id, eax
	}
	return id;
}

/** Fill buffer with unconstrained symbolic values. */
static _inline void s2e_make_symbolic(void* buf, int _size, const char* name)
{
    __s2e_touch_string(name);
    __s2e_touch_buffer(buf, _size);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     *__asm__ __volatile__(
     *   "pushl %%ebx\n"
     *   "movl %%edx, %%ebx\n"
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x03, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   "popl %%ebx\n"
     *   : : "a" (buf), "d" (size), "c" (name) : "memory"
     * );
	 *
	 */
	_asm {
		mov eax, buf
		mov edx, _size
		mov ecx, name
		
		push ebx
		mov ebx, edx

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x03
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		pop ebx

		// TODO: Not clear how to convert 'memory' constraint
	}
}

/** Fill buffer with unconstrained symbolic values without discarding concrete data. */
static _inline void s2e_make_concolic(void* buf, int _size, const char* name)
{
    __s2e_touch_string(name);
    __s2e_touch_buffer(buf, _size);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   "pushl %%ebx\n"
     *   "movl %%edx, %%ebx\n"
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x11, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   "popl %%ebx\n"
     *   : : "a" (buf), "d" (size), "c" (name) : "memory"
     * );
	 *
	 */
	_asm {
		mov eax, buf
		mov edx, _size
		mov ecx, name

		push ebx
		mov ebx, edx

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x11
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		pop ebx

		// TODO: Not clear how to convert 'memory' constraint
	}
}

/** Returns true if ptr points to symbolic memory */
static _inline int s2e_is_symbolic(void* _ptr, size_t _size)
{
    int result;
    __s2e_touch_buffer(_ptr, 1);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x04, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : "=a" (result) : "a" (size), "c" (ptr)
     * );
	 *
	 */
	_asm {
		mov eax, _size
		mov ecx, _ptr

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x04
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		mov result, eax
	}
    return result;
}

/** Concretize the expression. */
static _inline void s2e_concretize(void* buf, int _size)
{
    __s2e_touch_buffer(buf, _size);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   "pushl %%ebx\n"
     *   "movl %%edx, %%ebx\n"
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x20, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   "popl %%ebx\n"
     *   : : "a" (buf), "d" (size) : "memory"
     * );
	 *
	 */
	_asm {
		mov eax, buf
		mov edx, _size

		push ebx
		mov ebx, edx

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x20
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		pop ebx

		// TODO: Not clear how to convert 'memory' constraint
	}
}

/** Get example value for expression (without adding state constraints). */
static _inline void s2e_get_example(void* buf, int _size)
{
    __s2e_touch_buffer(buf, _size);
	/* 
	 * Original GCC inline assembly left for reference
	 *
	 *
     * __asm__ __volatile__(
     *   "pushl %%ebx\n"
     *   "movl %%edx, %%ebx\n"
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x21, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   "popl %%ebx\n"
     *   : : "a" (buf), "d" (size) : "memory"
     * );
	 *
	 */
	_asm {
		mov eax, buf
		mov edx, _size

		push ebx
		mov ebx, edx

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x21
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
		
		// Not clear how to convert 'memory' constraint
	}
}

/** Get example value for expression (without adding state constraints). */
/** Convenience function to be used in printfs */
static _inline unsigned s2e_get_example_uint(unsigned val)
{
    unsigned buf = val;
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   "pushl %%ebx\n"
     *   "movl %%edx, %%ebx\n"
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x21, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   "popl %%ebx\n"
     *   : : "a" (&buf), "d" (sizeof(buf)) : "memory"
     * );
	 *
	 */
	
	// sizeof cannot be called from within the _asm so
	// we call it and set bufsz before entering the _asm
	int bufsz = sizeof(buf);
	_asm {
		lea edx, buf
		mov edx, bufsz
		
		push ebx
		mov ebx, edx

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x21
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
		
		// TODO: Not clear how to convert 'memory' constraint
	}
    return buf;
}

/** Terminate current state. */
static _inline void s2e_kill_state(int status, const char* message)
{
    __s2e_touch_string(message);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   "pushl %%ebx\n"
     *   "movl %%edx, %%ebx\n"
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x06, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   "popl %%ebx\n"
     *   : : "a" (status), "d" (message)
     * );
	 *
	 */
	_asm {
		mov eax, status
		mov edx, message

		push ebx
		mov ebx, edx

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x06
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		pop ebx
	}
}

/** Disable timer interrupt in the guest. */
static _inline void s2e_disable_timer_interrupt()
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x50, 0x01, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     * );
	 *
	 */
	_asm{
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x50
		__emit 0x01
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Enable timer interrupt in the guest. */
static _inline void s2e_enable_timer_interrupt()
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x50, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     * );
	 *
	 */
	_asm {
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x50
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Disable all APIC interrupts in the guest. */
static _inline void s2e_disable_all_apic_interrupts()
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x51, 0x01, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     * );
	 *
	 */
	_asm {
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x51
		__emit 0x01
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Enable all APIC interrupts in the guest. */
static _inline void s2e_enable_all_apic_interrupts()
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x51, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     * );
	 *
	 */
	_asm{
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x51
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Get the current S2E_RAM_OBJECT_BITS configuration macro */
static _inline int s2e_get_ram_object_bits()
{
    int bits;
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x52, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : "=a" (bits)  : "a" (0)
     * );
	 *
	 */
	_asm {
		mov eax, 0

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x52
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		mov bits, eax
	}
    return bits;
}

/** Declare a merge point: S2E will try to merge
 *  all states when they reach this point.
 *
 * NOTE: This requires merge searcher to be enabled. */
static _inline void s2e_merge_point()
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x70, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     * );
	 *
	 */
	_asm {
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x70
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Open file from the guest.
 *
 * NOTE: This require HostFiles plugin. */
static _inline int s2e_open(const char* fname)
{
    int fd;
    __s2e_touch_string(fname);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0xEE, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : "=a" (fd) : "a"(-1), "b" (fname), "c" (0)
     * );
	 *
	 */
	_asm {
		mov eax, -1
		mov ebx, fname
		mov ecx, 0

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0xEE
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		mov fd, eax
	}
    return fd;
}

/** Close file from the guest.
 *
 * NOTE: This require HostFiles plugin. */
static _inline int s2e_close(int fd)
{
    int res;
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0xEE, 0x01, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : "=a" (res) : "a" (-1), "b" (fd)
     * );
	 *
	 */
	_asm {
		mov eax, -1
		mov ebx, fd

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0xEE
		__emit 0x01
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		mov eax, res
	}
    return res;
}

/** Read file content from the guest.
 *
 * NOTE: This require HostFiles plugin. */
static _inline int s2e_read(int fd, char* buf, int count)
{
    int res;
    __s2e_touch_buffer(buf, count);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   "pushl %%ebx\n"
     *   "movl %%esi, %%ebx\n"
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0xEE, 0x02, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   "popl %%ebx\n"
     *   : "=a" (res) : "a" (-1), "S" (fd), "c" (buf), "d" (count)
     * );
	 *
	 */
	__asm {
		mov eax, -1
		mov esi, fd
		mov ecx, buf
		mov edx, count

		push ebx
		mov ebx, esi

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0xEE
		__emit 0x02
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		pop ebx

		mov res, eax
	}
    return res;
}

/** Enable memory tracing */
static _inline void s2e_memtracer_enable()
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0xac, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     * );
	 *
	 */
	_asm {
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0xac
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Disable memory tracing */
static _inline void s2e_memtracer_disable()
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0xac, 0x01, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *);
	 *
	 */
	_asm {
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0xac
		__emit 0x01
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Raw monitor plugin */
/** Communicates to S2E the coordinates of loaded modules. Useful when there is
    no plugin to automatically parse OS data structures */
static _inline void s2e_rawmon_loadmodule(const char *name, unsigned loadbase, unsigned _size)
{
    __s2e_touch_string(name);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   "pushl %%ebx\n"
     *   "movl %%edx, %%ebx\n"
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0xAA, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   "popl %%ebx\n"
     *   : : "a" (name), "d" (loadbase), "c" (size)
     * );
	 *
	 */
	_asm {
		mov eax, name
		mov edx, loadbase
		mov ecx, _size

		push ebx
		mov ebx, edx

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0xAA
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		pop ebx
	}
}

// Unable to find win32 equivalent to __attribute__((packed))
typedef struct _s2e_opcode_module_config_t {
    uint32_t name;
    uint64_t nativeBase;
    uint64_t loadBase;
    uint64_t entryPoint;
    uint64_t size;
    uint32_t kernelMode;
} /*__attribute__((packed))*/ s2e_opcode_module_config_t;

/** Raw monitor plugin */
/** Communicates to S2E the coordinates of loaded modules. Useful when there is
    no plugin to automatically parse OS data structures */
static _inline void s2e_rawmon_loadmodule2(const char *name,
                                         uint64_t nativebase,
                                         uint64_t loadbase,
                                         uint64_t entrypoint,
                                         uint64_t size, unsigned kernelMode)
{
    s2e_opcode_module_config_t cfg;
    cfg.name = (uint32_t) name;
    cfg.nativeBase = nativebase;
    cfg.loadBase = loadbase;
    cfg.entryPoint = entrypoint;
    cfg.size = size;
    cfg.kernelMode = kernelMode;

    __s2e_touch_string(name);

	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0xAA, 0x02, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : : "c" (&cfg)
     * );
	 *
	 */
	_asm {
		lea ecx, cfg
	
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0xAA
		__emit 0x02
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** CodeSelector plugin */
/** Enable forking in the current process (entire address space or user mode only) */
static _inline void s2e_codeselector_enable_address_space(unsigned user_mode_only)
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0xAE, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : : "c" (user_mode_only)
     * );
	 *
	 */
	_asm {
		mov ecx, user_mode_only
		
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0xAE
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Disable forking in the specified process (represented by its page directory).
    If pagedir is 0, disable forking in the current process. */
static _inline void s2e_codeselector_disable_address_space(uint64_t pagedir)
{
	/* 
	 * Original GCC inline assembly left for reference
	 *
     *__asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0xAE, 0x01, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : : "c" (pagedir)
	 * );
	 *
	 */

	/* WARNING: The above register constraint seems to be invalid, a 64-bit
	 * usninged int is specified to be loaded into register 'c' (ecx) a
	 * 32-bit register. I provide the win32 code below, but the first 'mov'
	 * instruction is technically invalid
	 */

	/*
	_asm {
		mov ecx, pagedir
		
		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0xAE
		__emit 0x01
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
	*/
}

static _inline void s2e_codeselector_select_module(const char *moduleId)
{
    __s2e_touch_string(moduleId);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0xAE, 0x02, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : : "c" (moduleId)
     * );
	 *
	 */
	_asm {
		mov ecx, moduleId

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0xAE
		__emit 0x02
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

/** Programmatically add a new configuration entry to the ModuleExecutionDetector plugin */
static _inline void s2e_moduleexec_add_module(const char *moduleId, const char *moduleName, int kernelMode)
{
    __s2e_touch_string(moduleId);
    __s2e_touch_string(moduleName);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0xAF, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *       : : "c" (moduleId), "a" (moduleName), "d" (kernelMode)
     * );
	 *
	 */
	_asm {
		mov ecx, moduleId
		mov eax, moduleName
		mov edx, kernelMode

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0xAF
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00
	}
}

#ifdef USER_APP
/**
 * If processToRetrive == null,  get the name, load address, and
 * the size of the currently running process.
 *
 * If processToRetrive != null,  get the name, load address, and
 * the size of the specified module in the currently running process.
 *
 * The returned name is an absolute path to the program file.
 */
static _inline int s2e_get_module_info(const char *moduleToRetrieve,
                                char *name, size_t maxNameLength,
                                uint64_t *loadBase, uint64_t *size)
{
    FILE* maps;
	size_t start;
	size_t end;
    char executable;
    char line[256], path[128];
	const char* modName = NULL;
    int result = -1;

    if (moduleToRetrieve) {
        modName = strrchr(moduleToRetrieve, '/');
        if (modName == NULL) {
            modName = name;
        } else {
            ++modName;  // point to the first char after the slash
        }
    }

    maps = fopen("/proc/self/maps", "r");
    if (!maps) {
        return result;
    }

    while (fgets(line, sizeof(line), maps)) {
        if (sscanf(line, "%x-%x %*c%*c%c%*c %*x %*s %*d %127[^\n]", &start, &end, &executable, path) == 4) {
            if (modName) {
                if (executable == 'x' && strstr(path, modName) != NULL) {
                    *loadBase = start;
                    *size = end - start;
                    strncpy(name, path, maxNameLength);
                    result = 0;
                    break;
                }
            } else {
                if (!executable) {
                    continue;
                }

                //Found the module, get its data
                *loadBase = start;
                *size = end - start;
                strncpy(name, path, maxNameLength);
                result = 0;
                break;
            }
        }
    }

    fclose(maps);
    return result;
}

#endif

/* Kills the current state if b is zero */
static _inline void _s2e_assert(int b, const char *expression )
{
   if (!b) {
      s2e_kill_state(0, expression);
   }
}

#define s2e_assert(expression) _s2e_assert(expression, "Assertion failed: "  #expression)

/** Returns a symbolic value in [start, end) */
static _inline int s2e_range(int start, int end, const char* name) {
  int x = -1;

  if (start >= end) {
    s2e_kill_state(1, "s2e_range: invalid range");
  }

  if (start+1==end) {
    return start;
  } else {
    s2e_make_symbolic(&x, sizeof x, name);

    /* Make nicer constraint when simple... */
    if (start==0) {
      if ((unsigned) x >= (unsigned) end) {
        s2e_kill_state(0, "s2e_range creating a constraint...");
      }
    } else {
      if (x < start || x >= end) {
        s2e_kill_state(0, "s2e_range creating a constraint...");
      }
    }

    return x;
  }
}

/**
 *  Transmits a buffer of dataSize length to the plugin named in pluginName.
 *  eax contains the failure code upon return, 0 for success.
 */
static _inline int s2e_invoke_plugin(const char *pluginName, void *data, uint32_t dataSize)
{
    int result;
    __s2e_touch_string(pluginName);
    __s2e_touch_buffer(data, dataSize);
	/* 
	 * Original GCC inline assembly left for reference
	 *
     * __asm__ __volatile__(
     *   ".byte 0x0f, 0x3f\n"
     *   ".byte 0x00, 0x0b, 0x00, 0x00\n"
     *   ".byte 0x00, 0x00, 0x00, 0x00\n"
     *   : "=a" (result) : "a" (pluginName), "c" (data), "d" (dataSize) : "memory"
     * );
	 *
	 */
	_asm {
		mov eax, pluginName
		mov ecx, data
		mov edx, dataSize

		__emit 0x0f
		__emit 0x3f

		__emit 0x00
		__emit 0x0b
		__emit 0x00
		__emit 0x00

		__emit 0x00
		__emit 0x00
		__emit 0x00
		__emit 0x00

		mov result, eax
	}
    return result;
}
#endif
